package xyz.lychee.lagfixer.libs.adventure.text;

import java.util.List;
import java.util.Objects;
import xyz.lychee.lagfixer.libs.adventure.internal.Internals;
import xyz.lychee.lagfixer.libs.adventure.key.Key;
import xyz.lychee.lagfixer.libs.adventure.text.format.Style;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;

final class StorageNBTComponentImpl extends NBTComponentImpl<StorageNBTComponent, StorageNBTComponent.Builder> implements StorageNBTComponent {
   private final Key storage;

   static @NotNull StorageNBTComponent create(final @NotNull List<? extends ComponentLike> children, final @NotNull Style style, final String nbtPath, final boolean interpret, final @Nullable ComponentLike separator, final @NotNull Key storage) {
      return new StorageNBTComponentImpl(ComponentLike.asComponents(children, IS_NOT_EMPTY), (Style)Objects.requireNonNull(style, "style"), (String)Objects.requireNonNull(nbtPath, "nbtPath"), interpret, ComponentLike.unbox(separator), (Key)Objects.requireNonNull(storage, "storage"));
   }

   StorageNBTComponentImpl(final @NotNull List<Component> children, final @NotNull Style style, final String nbtPath, final boolean interpret, final @Nullable Component separator, final Key storage) {
      super(children, style, nbtPath, interpret, separator);
      this.storage = storage;
   }

   public @NotNull StorageNBTComponent nbtPath(final @NotNull String nbtPath) {
      return (StorageNBTComponent)(Objects.equals(this.nbtPath, nbtPath) ? this : create(this.children, this.style, nbtPath, this.interpret, this.separator, this.storage));
   }

   public @NotNull StorageNBTComponent interpret(final boolean interpret) {
      return (StorageNBTComponent)(this.interpret == interpret ? this : create(this.children, this.style, this.nbtPath, interpret, this.separator, this.storage));
   }

   public @Nullable Component separator() {
      return this.separator;
   }

   public @NotNull StorageNBTComponent separator(final @Nullable ComponentLike separator) {
      return create(this.children, this.style, this.nbtPath, this.interpret, separator, this.storage);
   }

   public @NotNull Key storage() {
      return this.storage;
   }

   public @NotNull StorageNBTComponent storage(final @NotNull Key storage) {
      return (StorageNBTComponent)(Objects.equals(this.storage, storage) ? this : create(this.children, this.style, this.nbtPath, this.interpret, this.separator, storage));
   }

   public @NotNull StorageNBTComponent children(final @NotNull List<? extends ComponentLike> children) {
      return create(children, this.style, this.nbtPath, this.interpret, this.separator, this.storage);
   }

   public @NotNull StorageNBTComponent style(final @NotNull Style style) {
      return create(this.children, style, this.nbtPath, this.interpret, this.separator, this.storage);
   }

   public boolean equals(final @Nullable Object other) {
      if (this == other) {
         return true;
      } else if (!(other instanceof StorageNBTComponent)) {
         return false;
      } else if (!super.equals(other)) {
         return false;
      } else {
         StorageNBTComponentImpl that = (StorageNBTComponentImpl)other;
         return Objects.equals(this.storage, that.storage());
      }
   }

   public int hashCode() {
      int result = super.hashCode();
      result = 31 * result + this.storage.hashCode();
      return result;
   }

   public String toString() {
      return Internals.toString(this);
   }

   public StorageNBTComponent.@NotNull Builder toBuilder() {
      return new BuilderImpl(this);
   }

   static class BuilderImpl extends AbstractNBTComponentBuilder<StorageNBTComponent, StorageNBTComponent.Builder> implements StorageNBTComponent.Builder {
      private @Nullable Key storage;

      BuilderImpl() {
      }

      BuilderImpl(final @NotNull StorageNBTComponent component) {
         super(component);
         this.storage = component.storage();
      }

      public StorageNBTComponent.@NotNull Builder storage(final @NotNull Key storage) {
         this.storage = (Key)Objects.requireNonNull(storage, "storage");
         return this;
      }

      public @NotNull StorageNBTComponent build() {
         if (this.nbtPath == null) {
            throw new IllegalStateException("nbt path must be set");
         } else if (this.storage == null) {
            throw new IllegalStateException("storage must be set");
         } else {
            return StorageNBTComponentImpl.create(this.children, this.buildStyle(), this.nbtPath, this.interpret, this.separator, this.storage);
         }
      }
   }
}
